import { InstrumentationBase, InstrumentationNodeModuleDefinition } from '@opentelemetry/instrumentation';
import { PinoInstrumentationConfig } from './types';
export declare class PinoInstrumentation extends InstrumentationBase<PinoInstrumentationConfig> {
    constructor(config?: PinoInstrumentationConfig);
    protected init(): InstrumentationNodeModuleDefinition[];
    private _callHook;
    private _getMixinFunction;
}
//# sourceMappingURL=instrumentation.d.ts.map